@@ -20,6 +20,9 @@ class TestDiscovery(unittest.TestCase):
 
     def test_discovery_with_broken(self):
         if "someapp" in settings.INSTALLED_APPS:
-            settings.INSTALLED_APPS = settings.INSTALLED_APPS + \
-                    ["xxxnot.aexist"]
-            self.assertRaises(ImportError, autodiscover)
+            installed_apps = list(settings.INSTALLED_APPS)
+            settings.INSTALLED_APPS = installed_apps + ["xxxnot.aexist"]
+            try:
+                self.assertRaises(ImportError, autodiscover)
+            finally:
+                settings.INSTALLED_APPS = installed_apps
