@@ -1,4 +1,4 @@
-import unittest
+import unittest2 as unittest
 
 from celery.worker import revoke
 
@@ -7,6 +7,6 @@ class TestRevokeRegistry(unittest.TestCase):
 
     def test_is_working(self):
         revoke.revoked.add("foo")
-        self.assertTrue("foo" in revoke.revoked)
+        self.assertIn("foo", revoke.revoked)
         revoke.revoked.pop_value("foo")
-        self.assertTrue("foo" not in revoke.revoked)
+        self.assertNotIn("foo", revoke.revoked)
