@@ -23,6 +23,39 @@ RUN = 0x0
 CLOSE = 0x1
 
 
+class QoS(object):
+    prev = None
+
+    def __init__(self, consumer, initial_value, logger):
+        self.consumer = consumer
+        self.logger = logger
+        self.value = SharedCounter(initial_value)
+
+        self.set(int(self.value))
+
+    def increment(self):
+        return self.set(self.value.increment())
+
+    def decrement(self):
+        return self.set(self.value.decrement())
+
+    def decrement_eventually(self):
+        self.value.decrement()
+
+    def set(self, pcount):
+        self.logger.debug("basic.qos: prefetch_count->%s" % pcount)
+        self.consumer.qos(prefetch_count=pcount)
+        self.prev = pcount
+        return pcount
+
+    def update(self):
+        return self.set(self.next)
+
+    @property
+    def next(self):
+        return int(self.value)
+
+
 class CarrotListener(object):
     """Listen for messages received from the broker and
     move them the the ready queue for task processing.
@@ -56,10 +89,10 @@ class CarrotListener(object):
         self.init_callback = init_callback
         self.logger = logger
         self.hostname = hostname or socket.gethostname()
+        self.initial_prefetch_count = initial_prefetch_count
         self.control_dispatch = ControlDispatch(logger=logger,
                                                 hostname=self.hostname,
                                                 listener=self)
-        self.prefetch_count = SharedCounter(initial_prefetch_count)
         self.event_dispatcher = None
         self.heart = None
         self._state = None
@@ -90,13 +123,9 @@ class CarrotListener(object):
         wait_for_message = self._detect_wait_method()(limit=None).next
         self.logger.debug("CarrotListener: Ready to accept tasks!")
 
-        prev_pcount = None
         while 1:
-            pcount = int(self.prefetch_count) # SharedCounter() -> int()
-            if not prev_pcount or pcount != prev_pcount:
-                task_consumer.qos(prefetch_count=pcount)
-                prev_pcount = pcount
-
+            if self.qos.prev != self.qos.next:
+                self.qos.update()
             wait_for_message()
 
     def on_task(self, task, eta=None):
@@ -119,11 +148,11 @@ class CarrotListener(object):
         if eta:
             if not isinstance(eta, datetime):
                 eta = parse_iso8601(eta)
-            self.prefetch_count.increment()
+            self.qos.increment()
             self.logger.info("Got task from broker: %s[%s] eta:[%s]" % (
                     task.task_name, task.task_id, eta))
             self.eta_schedule.enter(task, eta=eta,
-                                    callback=self.prefetch_count.decrement)
+                    callback=self.qos.decrement_eventually)
         else:
             self.logger.info("Got task from broker: %s[%s]" % (
                     task.task_name, task.task_id))
@@ -214,6 +243,10 @@ class CarrotListener(object):
         self.connection = self._open_connection()
         self.logger.debug("CarrotListener: Connection Established.")
         self.task_consumer = get_consumer_set(connection=self.connection)
+        # QoS: Reset prefetch window.
+        self.qos = QoS(self.task_consumer,
+                       self.initial_prefetch_count, self.logger)
+
         self.task_consumer.on_decode_error = self.on_decode_error
         self.broadcast_consumer = BroadcastConsumer(self.connection,
                                                     hostname=self.hostname)
